package client.performance;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.apache.log4j.Logger;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import server.ControllerUtil;
import server.EchoTestServer;
import client.BtNIOClient;

import com.sun.grizzly.Controller;
import com.sun.grizzly.ProtocolFilter;

import filter.PerformanceTestFilter;


public class ClientPerformanceTest {

	private static final Logger	LOG	= Logger.getLogger(ClientPerformanceTest.class);
	
	private Controller grizzlyController = new Controller();
	private ControllerUtil util = new ControllerUtil(grizzlyController);
	
	private static int serverPort = 5070;
	private static Controller.Protocol transportProtocol = Controller.Protocol.TCP;
	
	private EchoTestServer server;
	private BtNIOClient client;
	
	private int nmb_connections = 2500;
	private CountDownLatch latch = new CountDownLatch(nmb_connections);
	private ProtocolFilter applicationProtocolFilter = new PerformanceTestFilter(latch);
	
	@Before
	public void startController() throws IOException, InterruptedException
	{
		server = new EchoTestServer(grizzlyController, serverPort, transportProtocol);
		server.initServer();
		
		client = new BtNIOClient(grizzlyController);
		client.setApplicationProtocolFilter(applicationProtocolFilter);
		client.init(transportProtocol);
		
		util.startController();
	}
	
	@After
	public void stopController() throws InterruptedException, IOException
	{
		util.stopController();
	}
	
	@Test
	public void performanceTest1() throws UnknownHostException, IOException, InterruptedException
	{	
		List<Sender> senders = new ArrayList<Sender>();
		for(int i = 0; i<nmb_connections; i++)
		{
			BtNIOClient.Connector connector = client.getNewConnector();
			senders.add(new Sender(connector, serverPort));
		}
		
		ExecutorService executor = Executors.newFixedThreadPool(10);
		for(Sender sender: senders)
		{
			executor.execute(sender);
		}
		
		latch.await();
	}
	
	private class Sender implements Runnable
	{
		private BtNIOClient.Connector connector;
		private int serverPort;
		
		public Sender(BtNIOClient.Connector connector, int serverPort) {
			this.connector = connector;
			this.serverPort = serverPort;
		}
		@Override
		public void run() {
			try
			{
				connector.connect(new InetSocketAddress(InetAddress.getLocalHost(), serverPort));
				ByteBuffer outputByteBuffer1 = ByteBuffer.wrap(new String("\r\nPASS\r\n\0").getBytes());
				connector.send(outputByteBuffer1);
			}
			catch(IOException ex)
			{
				ex.printStackTrace();				
			}
		}
	}
}
