package server;

import java.io.IOException;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.concurrent.CountDownLatch;

import org.apache.log4j.Logger;

import com.sun.grizzly.Controller;
import com.sun.grizzly.ControllerStateListener;

public class ControllerUtil {

	private static final Logger	LOG	= Logger.getLogger(ControllerUtil.class);
	
	private Controller controller;
	
	private CountDownLatch controllerReady;
	private CountDownLatch controllerStopped;
	private long controllerStartTime;
	private long controllerStopTime;

	public Controller getController() {
		return controller;
	}

	public void setController(Controller controller) {
		this.controller = controller;
	}
	
	public ControllerUtil(Controller controller) {
		this.controller = controller;
		controllerReady = new CountDownLatch(1);
		controllerStopped = new CountDownLatch(1);
		
		controller.addStateListener(new GrizzlyControllerStateListenerAdapter());
	}
	
	private class GrizzlyControllerStateListenerAdapter implements ControllerStateListener
	{
		private Format dateFormatter = new SimpleDateFormat("yyyy.MM.dd G 'at' HH:mm:ss.SSS z");
		
		@Override
		public void onException(Throwable e) {
			e.printStackTrace();
		}
		
		@Override
		public void onReady() {
			controllerReady.countDown();
			if(LOG.isTraceEnabled())
			{
				LOG.trace("Grizzly Controller is ready");
			}
		}
		
		@Override
		public void onStarted() {
			controllerStartTime = System.currentTimeMillis();
				
			if(LOG.isTraceEnabled())
			{
				LOG.trace("Grizzly Controller is started");
				Date startDate = new Date(controllerStartTime);
				LOG.trace("Grizzly Controller started on: " + dateFormatter.format(startDate));
			}
		}
		
		@Override
		public void onStopped() {
			controllerStopTime = System.currentTimeMillis();
			controllerStopped.countDown();
			if(LOG.isTraceEnabled())
			{
				LOG.trace("Grizzly Controller is stopped");
				Date stopDate = new Date(controllerStopTime);
				LOG.trace("Grizzly Controller stopped on: " + dateFormatter.format(stopDate));
				Double durationInSeconds = (double)(controllerStopTime - controllerStartTime)/1000L;
				LOG.trace("Grizzly Controller was up during: " + durationInSeconds + " (seconds)");
			}
		}
	}
	
	public void startController() throws InterruptedException
	{
		new Thread(controller).start();
		controllerReady.await();

	}
	
	public void stopController() throws InterruptedException, IOException
	{
		controller.stop(true);
		controllerStopped.await();
	}
}
