package server;

import org.apache.log4j.Logger;

import com.sun.grizzly.Controller;
import com.sun.grizzly.DefaultProtocolChain;
import com.sun.grizzly.Pipeline;
import com.sun.grizzly.ProtocolChain;
import com.sun.grizzly.ProtocolChainInstanceHandler;
import com.sun.grizzly.TCPSelectorHandler;
import com.sun.grizzly.UDPSelectorHandler;
import com.sun.grizzly.filter.EchoFilter;
import com.sun.grizzly.filter.ReadFilter;

public class EchoTestServer {
	private static final Logger	LOG	= Logger.getLogger(EchoTestServer.class);
	
	private int serverPort;
	private Controller.Protocol transportProtocol;
	private Controller controller;
	private Pipeline serverSelectorPipeline;

	public void setController(Controller controller) {
		this.controller = controller;
	}

	public void setServerSelectorPipeline(Pipeline serverSelectorPipeline) {
		this.serverSelectorPipeline = serverSelectorPipeline;
	}

	public EchoTestServer(Controller controller, int port, Controller.Protocol protocol) {
		this.controller = controller;
		this.serverPort = port;
		this.transportProtocol = protocol;
	}

	public void initServer() 
	{
		TCPSelectorHandler selector = null;
		if (transportProtocol == Controller.Protocol.TCP) {
			selector = new TCPSelectorHandler();
			
		} else if (transportProtocol == Controller.Protocol.UDP) {
			selector = new UDPSelectorHandler();
		}
		selector.setPort(serverPort);
		if(serverSelectorPipeline != null)
		{
			selector.setPipeline(serverSelectorPipeline);
		}
		
		controller.addSelectorHandler(selector);
		
		ProtocolChainInstanceHandler pciHandler = new ProtocolChainInstanceHandler() {

			final private ProtocolChain protocolChain = new DefaultProtocolChain();

			public ProtocolChain poll() {
				return protocolChain;
			}

			public boolean offer(ProtocolChain instance) {
				return true;
			}
		};
		selector.setProtocolChainInstanceHandler(pciHandler);

		ProtocolChain protocolChain = pciHandler.poll();
		protocolChain.addFilter(new ReadFilter());
		protocolChain.addFilter(new EchoFilter());
	}
}
